include("functions.jl")

# Const and type declarations didn't do much to make the code speedier,
# but they added a lot of verbosity to the code, so I removed those again.

ITERATIONS = 100
DENSITY = 1000
x_min, x_max = -2.68, 1.32
y_min, y_max = -1.5, 1.5

x,y = meshgrid(LinRange(x_min, x_max, DENSITY), LinRange(y_min, y_max, DENSITY))
c = x + (y)im
z = copy(c)
fractal = fill(255,DENSITY,DENSITY)

a = time()
z = copy(c)
fractal = fill(255,DENSITY,DENSITY)
mask = [abs(α) ≤ 10 for α in z] # initial fill
# for loop contents improved by devin.jl
#ITERATIONS=10
for n in 1:ITERATIONS
	#println("Iteration ", n)
	@. mask = abs(z) ≤ 10
	z[mask] .^= 2
	z[mask] .+= c[mask]
	brightness = 254 * (n-1) ÷ ITERATIONS
	@. fractal[abs(z) > 10 && fractal == 255] = brightness
end
b = time()
println(round(b-a,digits=1), "s")

a = time()
z = copy(c)
fractal = fill(255,DENSITY,DENSITY)
# Clemonthyme https://gist.github.com/Clemapfel/a04c5ec73f5f43b69c9097e068c15c7d
for n in 1:ITERATIONS
	#println("Iteration ", n)
	for i in 1:length(z)
		if abs(z[i]) <= 10
			z[i] ^= 2
			z[i] += c[i]
		#end
		#if abs(z[i]) > 10 && fractal[i] == 255
		# replaced by Jacob with elseif
		elseif fractal[i] == 255
			fractal[i] = 254 * (n - 1) ÷ ITERATIONS
		end
	end
end
b = time()
println(round(b-a,digits=1), "s")

logfractal = log.(fractal) #devin.jl

println("Saving...")
savetxt("fractal_jl.dat", logfractal)
savetxt("coord_jl.dat", [ x_min, x_max, y_min, y_max ] )


exit()
#
# functions.jl
#

# meshgrid function by Chris Rackauckas, 2016
# https://groups.google.com/g/julia-users/c/83Pfg9HGhGQ/m/9G_0wi-GBQAJ
function meshgrid(a,b)
	x = a' .* ones(length(b))
	y = ones(length(a))' .* b
	return x,y
end

# stylistic improvement by Jacob
savetxt(filename, array) = open(filename) do io
	for row in eachrow(array)
		line = replace(string(row), "[" => "", "]" => "", "," => "")
		println(io, line)
	end
end
